unit IntroDemoUnit;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, yapi, ExtCtrls, Db, DBTables,printers;

type
  TForm1 = class(TForm)
    TextMemo: TMemo;
    Button1: TButton;
    Paper: TyapiPaper;
    Header: TyapiHeaderFooter;
    yapiHeaderFooter1: TyapiHeaderFooter;
    AthenaImage: TyapiImage;
    yapiTab1: TyapiTab;
    BulletImage: TyapiImage;
    Title: TyapiText;
    Heading: TyapiText;
    Body: TyapiText;
    GridHeading: TyapiGridText;
    GridBody: TyapiGridText;
    GridColour: TyapiGridText;
    tenPt: TyapiText;
    Form: TyapiImage;
    Fixed: TyapiText;
    PaintBox: TyapiPaintBox;
    PrintDialog: TPrintDialog;
    SourceMemo: TMemo;
    procedure Button1Click(Sender: TObject);
    procedure PaintBoxPaint(Canvas: TCanvas; left, top, right,
      bottom: Integer; Ref: TObject; PaintboxNumber, page: Integer;
      Printer: Boolean);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
    lineindex:Integer;
    sinedata,cosdata:array[0..19] of double;
    procedure PutText;
    procedure PlaceGrid;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.PutText;
{ this procedure puts text from the TextMemo component
 (which has the textual content of the report) on to yapi.
 It operates with a simple markup language
}
var s:string;
i:Integer;
mark:char;
begin
  with textmemo do begin
     i:=lines.count;
     while lineindex<i do begin
        s:=lines[lineindex];
        if (length(s)>1) and (s[1]='-') then begin
           mark:=lines[lineindex][2];
           case mark of
              'H' : // Print a sub heading line
                       Heading.writeln(pchar(s)+3);
              'S' : begin
                       // exit with linindex pointing to the next line
                       inc(lineindex);
                       exit;
                    end;
              'B' : begin
                      body.write('     '); // simple indent
                      BulletImage.write;   // write out our bullet - an image
                      body.write(' ');     // spacing between bullet and text
                      body.writeln(pchar(s)+3);
                      body.writeln;        // next line
                    end;
              'P' : begin
                      paper.newpage;
                    end;

           end;
        end else begin
           body.writeln(lines[lineindex]);
        end;
        inc(lineindex);
     end;
  end;
end;

procedure TForm1.PlaceGrid;
{ this procedure puts the data in the grid on the report }
var i:Integer;
tot1,tot2,v:double;
begin
   paper.settabspacing([gridbody.getWidth('Value   '),
                        gridbody.getWidth('0.00000'),
                        gridbody.getWidth('0.00000')],80);
   gridcolour.writecentre('Value',0);
   gridcolour.writecentre('Sine',1);
   gridcolour.writecentre('Cos',2);
   gridcolour.writeln;
   tot1:=0;
   tot2:=0;
   for i:=0 to 19 do begin
      gridbody.writecentre(inttostr(i*20),0);
      v:=sin(i*20*3.14159/180);
      sinedata[i]:=v;
      tot1:=tot1+v;
      gridbody.writeright(format('%.2f ',[v]),1);
      v:=cos(i*20*3.14159/180);
      cosdata[i]:=v;
      tot2:=tot2+v;
      gridbody.writeright(format('%.2f ',[v]),2);
      gridbody.writeln;
   end;
   tenPt.write('Totals');
   gridbody.writeright(format('%.2f ',[tot1]),1);
   gridbody.writeright(format('%.2f ',[tot2]),2);
   gridbody.writeln;
   paper.resettab;
end;

procedure TForm1.Button1Click(Sender: TObject);
// this is the button that does all the printing
var
i:Integer;
begin
   paper.clear;
   lineindex:=0;
   title.writecentre('Yapi',0);     // page title
   AthenaImage.writeattab(1);       // Athena bit map on top of paper.
   title.writeln;                   // do some vertical spacing with empty writeln
   title.writeln;                   // spacing
   title.writeln;                   // spacing
   puttext;                         // place text from the TextMemo onto the printer
   paper.newpage;                   // form feed on the printer
   Gridheading.writeln('     Working with Grids     '); // Shadowed title
   puttext;                         // more text from the TextMemo onto the printer
   placegrid;                       // Put the grid onto the paper
   puttext;                         // more text from the TextMemo onto the printer
   body.writeln;                    // spacing
   body.writeln;                    // spacing
   paintbox.writeln;                // Put the paintbox onto the printer
   paper.newpage;                   // form feed on the printer
   puttext;                         // more text from the TextMemo onto the printer
   form.writeln;                    // Put the screen capture of this form onto the printer
   body.writeln;                    // spacing
   puttext;                         // more text from the TextMemo onto the printer
   body.writeln;                    // spacing
   for i:=0 to sourcememo.lines.count-1 do         // put in onthe paper using a fixed font
     fixed.writeln(sourcememo.lines[i]);
   paper.preview;                   // Print preview and printing
end;

procedure TForm1.PaintBoxPaint(Canvas: TCanvas; left, top, right,
  bottom: Integer; Ref: TObject; PaintboxNumber, page: Integer;
  Printer: Boolean);
  { This is an event for filling the paintbox.
    It takes the two arrays of sin and cos and plots them. This event will be used
    whenever the graph needs to be displayed - either at the preview time or
    at print time.
    To simplify grahics programing there are routines to convert from a virtual space
    which can be convientient and "fixed" is size and space to the required coordinates
  }
  const
     // define the virtual space in ral coordinates
     MinX=-1;    // left hand side of plot has virtual coordinate of -1
     MinY=-1.1;  // Bottom of plot of plot has virtual coordinate of  -1.1
     MaxX=21;    // Right hand side of Plot has virtual coordinate of  -21
     MaxY=1.1;   // Top of Plot is has virtual coordinate of  1.1
  var i:Integer;
  function getx(val:double):Integer;      // scale coordinates to image space
  begin
     result:=left+round((val-Minx)/(MaxX-Minx)*(right-left));
  end;
  function gety(val:double):Integer;      // scale coordinates to image space
  begin
     result:=bottom-round((val-Miny)/(MaxY-Miny)*(bottom-top));
  end;
  procedure drawline(x1,y1,x2,y2:double); // draws a line in virtual space
  begin
      canvas.moveto(getx(x1),gety(y1));
      canvas.lineto(getx(x2),gety(y2));
  end;
begin
  Canvas.Brush.Color := clBlack;
  canvas.FrameRect(bounds(left,top,right-left,bottom-top));
  Canvas.Brush.Color := clwhite;
  // draw xaxis - use coordinates in virtuyal space
  drawline( 0,0 , 20,0 );
  // draw yaxis - use coordinates in virtual space
  drawline( 0,-1 , 0,1 );
  for i:=0 to 18 do begin
      // draw data in virtual space
      canvas.pen.color:=clBlue;
      drawline( i,sinedata[i] , 1+i,sinedata[i+1] );
      canvas.pen.color:=clRed;
      drawline( i,cosdata[i] , 1+i,cosdata[i+1] );
  end;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  textmemo.visible:=false;
  sourcememo.visible:=false;
end;

end.
